 
#ifndef __CONSENSUS_ESTIMATOR_H
#define __CONSENSUS_ESTIMATOR_H
#include "dsPIC_XBeePackets.h"
#include "e_motors_swarm.h"
#include <math.h>
#include "main.h"
#include "global_vars.h"


float num_agents=0;  // number of agents you've received data from.

//Estimator Gains	
float KP=0.7; 
float KI=0.1;
float GAMMA=0.05;

//Data arrays
//DATA_ARRAY_LENGTH and NUM_DATA_SETS are in main.h
//volatile float x_i[NUM_DATA_SETS]={
//	(START_COORD_X+POINT_OFFSET), 
//	(START_COORD_Y), 
//	(START_COORD_X+POINT_OFFSET)*(START_COORD_X+POINT_OFFSET),
//	(START_COORD_X+POINT_OFFSET)*(START_COORD_Y),
//	(START_COORD_Y+POINT_OFFSET)*(START_COORD_Y)};	//x, decision variables
//	
//volatile float u_i[NUM_DATA_SETS]={
//	(START_COORD_X+POINT_OFFSET),
//	(START_COORD_Y),
//	(START_COORD_X+POINT_OFFSET)*(START_COORD_X+POINT_OFFSET),
//	(START_COORD_X+POINT_OFFSET)*(START_COORD_Y),
//	(START_COORD_Y+POINT_OFFSET)*(START_COORD_Y)};	//u, Controller inputs


extern volatile float x_i[NUM_DATA_SETS];	//x, current_estimate/decision variables
extern volatile float u_i[NUM_DATA_SETS];	//u, Controller inputs
extern volatile float w_i[NUM_DATA_SETS];	//w, internal estimator state
extern volatile float x_sum[NUM_DATA_SETS];	//running sums
extern volatile float w_sum[NUM_DATA_SETS];





	
//flags for received messages, so we can keep track of whom we have received messages from.
//31 robots maximum right now (0 is reserved for base station)
volatile union MSG_RX_FLAGS {
	unsigned long allFlags;//32 bits
	unsigned char flagBytes[4];
	struct{
		unsigned flag_0:1;
		unsigned flag_1:1;
		unsigned flag_2:1;
		unsigned flag_3:1;
		unsigned flag_4:1;
		unsigned flag_5:1;
		unsigned flag_6:1;
		unsigned flag_7:1;
		unsigned flag_8:1;
		unsigned flag_9:1;
		unsigned flag_10:1;
		unsigned flag_11:1;
		unsigned flag_12:1;
		unsigned flag_13:1;
		unsigned flag_14:1;
		unsigned flag_15:1;
		unsigned flag_16:1;
		unsigned flag_17:1;
		unsigned flag_18:1;
		unsigned flag_19:1;
		unsigned flag_20:1;
		unsigned flag_21:1;
		unsigned flag_22:1;
		unsigned flag_23:1;
		unsigned flag_24:1;
		unsigned flag_25:1;
		unsigned flag_26:1;
		unsigned flag_27:1;
		unsigned flag_28:1;
		unsigned flag_29:1;
		unsigned flag_30:1;
		unsigned flag_31:1;
	};
}MsgRxFlags;

//returns value of message flag
unsigned char getMsgRxFlag(unsigned int flagNum){
	unsigned int quotient=flagNum/8;  //find the byte
	unsigned int remainder=flagNum%8; //find the bit in the byte
	unsigned char temp;
	if (flagNum > 31){ //number is out of range
		return -1;
	}
	temp = MsgRxFlags.flagBytes[quotient]&=(0b00000001<<remainder);
	if (temp == 0){
		return 0;
	}
	else {
		return 1;
	}
}

//sets value of message flag
unsigned char setMsgRxFlag(unsigned int flagNum, unsigned int value){
	unsigned int quotient=flagNum/8;
	unsigned int remainder=flagNum%8;
	if (flagNum > 31){
		return -1;
	}
	if (value==0){
		MsgRxFlags.flagBytes[quotient]&=(0b11111110<<remainder);
	}
	else{
		MsgRxFlags.flagBytes[quotient]|=(0b00000001<<remainder);	
	}
	return 1;
}

void resetMsgRxFlags(void){
	MsgRxFlags.allFlags = 0;
}	

//adds new data to runnning sum x_sum.
//The data in the packet is structured: x_1 w_1 x_2 w_2 x_3 w_3 etc.
void addConsensusData(void){
//	extern volatile struct IN_PACKET InPacket;


	extern volatile struct IN_PACKET InPacket;

	int k;
	int j=0;
	for (k=0; k<DATA_ARRAY_LENGTH; ){
		x_sum[j]+=x_i[j]-InPacket.data[k].dataFloat; 	//add (x_i - x_j) to x_sum
		k++;
		w_sum[j]+=w_i[j]-InPacket.data[k].dataFloat;
		j++;
		k++;
	}
	num_agents++;  //keep track of number of agents you've received data from.
}

//Calculate a new estimate
void calcNewEstimate(void){
	int j;
	
	//if no messages were received, set num_agents = 1 to avoid div-by-zero error. 
	//x_i and w_i would be zero in that case anyway, so no difference.
	if (MsgRxFlags.allFlags==0){
		num_agents = 1;
	}
		
    e_get_configuration(&robotX, &robotY, &robotTheta);
 	u_i[0]=robotX; 
    u_i[1]=robotY;  
	u_i[2]=SQUARE(u_i[0]-x_i[0]);
	u_i[3]=(u_i[0]-x_i[0])*(u_i[1]-x_i[1]);
	u_i[4]=SQUARE(u_i[1]-x_i[1]);

	for (j=0; j<NUM_DATA_SETS; j++){
		x_i[j] = x_i[j] + GAMMA*(u_i[j]-x_i[j]) - (KP/num_agents)*x_sum[j] + (KI/num_agents)*w_sum[j];
		w_i[j] = w_i[j] - (KI/num_agents)*x_sum[j];
		x_sum[j]=0;	
		w_sum[j]=0;
	}
	num_agents = 0;
}	
	

#endif

