// functions for wheel_speed_coordinator. pulled out the h file
// by Sam Bobb. August 2009

#include "wheel_speed_coordinator.h"
#include <math.h>
#include "e_motors_swarm.h"
#include "global_vars.h"
//#include "main.h"

//#include "PI_consensus_estimator.h"
#include "e_led.h"

//GLOBAL VARIABLES//////////////////////////////////////////

extern float MAX_WHEEL_V_TICKS; //Max velocity; the larger this is, the less time you have
								 //to do other calculations.  Limit is probably around 750

extern float deadband;  //deadband value for low motor velocity
extern float COMMR;  //Max communication radius


static float k[5] = {1.0,1.0,1e-5,2e-5,1e-5};	//controller gains
//volatile static float goal[5] = {-100.0, -300.0, 160000.0, 40000.0, 40000.0};	//goal statistics x and y first moments, 3 central second moments
volatile static float goal[5];

//variables for obstacle avoidance

static volatile float nx=10000.0;	  // nearest neighbor's center point position, big number so that make sures nx,ny is initialized by the first packet it receives
static volatile float ny=10000.0; 
static volatile int nid = 100;      // id of the nearest neighbor, assume NOBODY will ever be 100
static volatile int nflag =0;       // used just for initialization

extern float SAFEDIST;	//Distance in mm at which obstacle avoidance will start running.
extern float MINDIST;	//Distance in mm at which obstacle avoidance will dominate the robot's motion.

extern float u_x_ideal;
extern float u_y_ideal;
//////////////////////////////////////////////////////////

extern float x_motion_integral;
extern float y_motion_integral;


extern volatile float x_i[NUM_DATA_SETS];	//x, current_estimate/decision variables



void wheelSpeed(int *vL, int *vR){	

	float faster_wheel_speed; //speed of the faster wheel
	float uX_R, uY_R; //motion vector in robot frame.
	float u_x, u_y;   //motion vector in world frame
	float u_mag;	  //magnitude of motion vector
	
	float CTheta;	//cosine of theta (orientation of robot)
	float STheta;	//sine of theta
	
	float vL_f, vR_f; //wheel velocities

	float robotX;  //Robot's position/orientation
	float robotY;
	float robotTheta;

	float robotFrontX,robotFrontY; //The x,y coords of the front reference point of the robot
	
	float n_vec_x;  //vector to nearest neighbor in world frame
	float n_vec_y;
	float n_mag;	//distance of the nearest neighbor

//	//peng, 9/10
//	
//	static float newgoal[5];
//	newgoal[0]=goal[0];
//	newgoal[1]=goal[1];
//	newgoal[2]=goal[2]+goal[0]*goal[0];
//	newgoal[3]=goal[3]+goal[0]*goal[1];
//	newgoal[4]=goal[4]+goal[1]*goal[1];
	//end change

	e_get_configuration_front(&robotFrontX, &robotFrontY, &robotTheta);	
	CTheta = cos(robotTheta);  //may use sin and cosine of theta more than once, pre-calculate to save time
	STheta = sin(robotTheta);

////////////////////////////////////////////////////////	
//	  MATLAB CODE:
//    cGains = diag([1.0, 1.0, 0.000005,0.00005,0.000005]);
//    controller = [1 0 2*(frontpos(1)-x_i(1))   (frontpos(2)-x_i(2))   0    ;
//                  0 1 0 (frontpos(1)-x_i(1)) 2*(frontpos(2)-x_i(2))]  ;
//
//    u_vel = controller*cGains*(newgoal-x_i);
////////////////////////////////////////////////////////

	//calculate desired vector with controller

	u_x = k[0]*(goal[0]-x_i[0])\
			+ 2*k[2]*(robotFrontX-x_i[0])*(goal[2]-x_i[2])\
			  + k[3]*(robotFrontY-x_i[1])*(goal[3]-x_i[3]);
			
    u_y = k[1]*(goal[1]-x_i[1])\
			  + k[3]*(robotFrontX-x_i[0])*(goal[3]-x_i[3])\
			+ 2*k[4]*(robotFrontY-x_i[1])*(goal[4]-x_i[4]); 



//	u_x = 200 - robotFrontX;
			
//    u_y = 200 - robotFrontY; 
	
	u_x_ideal = u_x;	//save ideal vector for logging
	u_y_ideal = u_y;
	
	//Obstacle Avoidance, by Mike 12/22/08
	e_get_configuration(&robotX, &robotY, &robotTheta);
	n_vec_x = nx-robotX;
	n_vec_y = ny-robotY;
	n_mag = sqrt(SQUARE(n_vec_x)+SQUARE(n_vec_y));		
	if (n_mag < SAFEDIST){
		float scale;
		float ratio;
		e_set_led(5,1);
		u_mag = sqrt(SQUARE(u_x)+SQUARE(u_y));
		scale = u_mag/n_mag;
		ratio = (n_mag-MINDIST)/(SAFEDIST);
		u_x = ratio*u_x - (1.0-ratio)*(n_vec_x*scale);
		u_y = ratio*u_y - (1.0-ratio)*(n_vec_y*scale);
	}	
	else{
		e_set_led(5,0);
	}	
	
	//end obstacle avoidance

	//put in deadband to stop robot when velocity vector is small
	u_mag = sqrt(SQUARE(u_x)+SQUARE(u_y));
	if (u_mag < deadband){
		u_x=0.0;
		u_y=0.0;
	}
	
	// transform velocity vector into the robot's local coordinates
	uX_R = CTheta*u_x + STheta*u_y;    
	uY_R = -STheta*u_x + CTheta*u_y;
	
	// translate velocity vector into wheel speeds
	vL_f = (uX_R-(uY_R*WHEELBASE/POINT_OFFSET));
    vR_f = (uX_R+(uY_R*WHEELBASE/POINT_OFFSET));


    //if either wheel velocity exceeds the maximum velocity, scale it down
    if ( fabs(vL_f)> MAX_WHEEL_V_TICKS || fabs(vR_f)> MAX_WHEEL_V_TICKS)
    	{
	    //determine which wheel is faster, and note the velocity
		faster_wheel_speed = (fabs(vL_f) > fabs(vR_f)? vL_f:vR_f );
	 	//use the larger wheel velocity as the divisor so that the faster
	 	//wheel goes at maximum speed
    	vL_f=(vL_f*MAX_WHEEL_V_TICKS)/fabs(faster_wheel_speed); 
    	vR_f=(vR_f*MAX_WHEEL_V_TICKS)/fabs(faster_wheel_speed);
    }
	*vL = (int)(vL_f);	//these values are returned
	*vR = (int)(vR_f);
}	



void wheelSpeedSingleBot(float gotox, float gotoy, int *vL, int *vR){	

	float faster_wheel_speed; //speed of the faster wheel
	float uX_R, uY_R; //motion vector in robot frame.
	float u_x, u_y;   //motion vector in world frame
	float u_mag;	  //magnitude of motion vector
	
	float CTheta;	//cosine of theta (orientation of robot)
	float STheta;	//sine of theta
	
	float vL_f, vR_f; //wheel velocities

	float robotX;  //Robot's position/orientation
	float robotY;
	float robotTheta;

	float robotFrontX,robotFrontY; //The x,y coords of the front reference point of the robot
	
	float n_vec_x;  //vector to nearest neighbor in world frame
	float n_vec_y;
	float n_mag;	//distance of the nearest neighbor

//	//peng, 9/10
//	
//	static float newgoal[5];
//	newgoal[0]=goal[0];
//	newgoal[1]=goal[1];
//	newgoal[2]=goal[2]+goal[0]*goal[0];
//	newgoal[3]=goal[3]+goal[0]*goal[1];
//	newgoal[4]=goal[4]+goal[1]*goal[1];
	//end change

	e_get_configuration_front(&robotFrontX, &robotFrontY, &robotTheta);	
	CTheta = cos(robotTheta);  //may use sin and cosine of theta more than once, pre-calculate to save time
	STheta = sin(robotTheta);

////////////////////////////////////////////////////////	
//	  MATLAB CODE:
//    cGains = diag([1.0, 1.0, 0.000005,0.00005,0.000005]);
//    controller = [1 0 2*(frontpos(1)-x_i(1))   (frontpos(2)-x_i(2))   0    ;
//                  0 1 0 (frontpos(1)-x_i(1)) 2*(frontpos(2)-x_i(2))]  ;
//
//    u_vel = controller*cGains*(newgoal-x_i);
////////////////////////////////////////////////////////

	//calculate desired vector with controller

/*
	u_x = k[0]*(goal[0]-x_i[0])\
			+ 2*k[2]*(robotFrontX-x_i[0])*(goal[2]-x_i[2])\
			  + k[3]*(robotFrontY-x_i[1])*(goal[3]-x_i[3]);
			
    u_y = k[1]*(goal[1]-x_i[1])\
			  + k[3]*(robotFrontX-x_i[0])*(goal[3]-x_i[3])\
			+ 2*k[4]*(robotFrontY-x_i[1])*(goal[4]-x_i[4]); 
*/


	// the reduced, hacked together version of the motion controller
	// use to make a single bot go to a single point. 
	// should be replaced with a 
	// "go to this point at a certain speed, 
	// check if there yet
	// yes -> done
	// no -> keep going"
	// algorithm, such as that in NUtest.c
    u_x = 30*(gotox - robotFrontX);
    u_y = 30*(gotoy - robotFrontY);
	
	
	u_x_ideal = u_x;	//save ideal vector for logging
	u_y_ideal = u_y;
	
	//Obstacle Avoidance, by Mike 12/22/08
	e_get_configuration(&robotX, &robotY, &robotTheta);
	n_vec_x = nx-robotX;
	n_vec_y = ny-robotY;
	n_mag = sqrt(SQUARE(n_vec_x)+SQUARE(n_vec_y));		
	if (n_mag < SAFEDIST){
		float scale;
		float ratio;
		e_set_led(5,1);
		u_mag = sqrt(SQUARE(u_x)+SQUARE(u_y));
		scale = u_mag/n_mag;
		ratio = (n_mag-MINDIST)/(SAFEDIST);
		u_x = ratio*u_x - (1.0-ratio)*(n_vec_x*scale);
		u_y = ratio*u_y - (1.0-ratio)*(n_vec_y*scale);
	}	
	else{
		e_set_led(5,0);
	}	
	
	//end obstacle avoidance

	//put in deadband to stop robot when velocity vector is small
	u_mag = sqrt(SQUARE(u_x)+SQUARE(u_y));
	if (u_mag < deadband){
		u_x=0.0;
		u_y=0.0;
	}
	
	// transform velocity vector into the robot's local coordinates
	uX_R = CTheta*u_x + STheta*u_y;    
	uY_R = -STheta*u_x + CTheta*u_y;
	
	// translate velocity vector into wheel speeds
	vL_f = (uX_R-(uY_R*WHEELBASE/POINT_OFFSET));
    vR_f = (uX_R+(uY_R*WHEELBASE/POINT_OFFSET));


    //if either wheel velocity exceeds the maximum velocity, scale it down
    if ( fabs(vL_f)> MAX_WHEEL_V_TICKS || fabs(vR_f)> MAX_WHEEL_V_TICKS)
    	{
	    //determine which wheel is faster, and note the velocity
		faster_wheel_speed = (fabs(vL_f) > fabs(vR_f)? vL_f:vR_f );
	 	//use the larger wheel velocity as the divisor so that the faster
	 	//wheel goes at maximum speed
    	vL_f=(vL_f*MAX_WHEEL_V_TICKS)/fabs(faster_wheel_speed); 
    	vR_f=(vR_f*MAX_WHEEL_V_TICKS)/fabs(faster_wheel_speed);
    }
	*vL = (int)(vL_f);	//these values are returned
	*vR = (int)(vR_f);
}





int abs(int input)
	{
	if (input<0)
		{
		input= -input;
		}
	return input;
	}

float fabs(float input)
	{
	if (input<0)
		{
		input= -input;
		}
	return input;
	}

